package de.fhdw.wtf.tooling.editor;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;

import de.fhdw.wtf.common.exception.editor.EditorMarker;

/**
 * A class which allows the Editor to Display Error Markers on Files in the users Eclipse.
 * 
 */
public class MarkerWriter {
	
	public static final String ABSTRACT_OPERATIONS_MARKER = "wtfeditor.editors.abstractOperationsMarker";
	public static final String BASETYPE_INHERITANCE_MARKER = "wtfeditor.editors.basetypeInheritanceMarker";
	public static final String CYCLIC_INHERITANCE_MARKER = "wtfeditor.editors.cyclicInheritanceMarker";
	public static final String DOUBLE_ATTRIBUTENAME_MARKER = "wtfeditor.editors.doubleAttributenameMarker";
	public static final String INVALID_ATTRIBUTE_MODIFIER_MARKER = "wtfeditor.editors.invalidAttributeModifierMarker";
	public static final String INVALID_OVERLOADING_MARKER = "wtfeditor.editors.invalidOverloadingMarker";
	public static final String INVALID_PRODUCT_MARKER = "wtfeditor.editors.invalidProductMarker";
	public static final String PARSER_MARKER = "wtfeditor.editors.parserMarker";
	public static final String REFERENCER_MARKER = "wtfeditor.editors.referencerMarker";
	
	public static final String UNDEFINED_MARKER = "wtfeditor.editors.undefinedMarker";
	
	/**
	 * This Method writes an Eclipse Error Marker on a given File.
	 * 
	 * @param file
	 *            A File, where the Marker will be placed. Must implement IResource
	 * @param line
	 *            The Line in which the parker should be placed.
	 * @param start
	 *            The column where the marker error should start.
	 * @param end
	 *            The column where marker error should end.
	 * @param message
	 *            The Message given as String, which will be showed by the Marker.
	 */
	public static void writeErrorMarker(final IResource file,
			final int line,
			final int start,
			final int end,
			final String message,
			final EditorMarker editorMarker) {
		
		String markerName;
		
		switch (editorMarker) {
		case ABSTRACT_OPERATIONS_MARKER:
			markerName = ABSTRACT_OPERATIONS_MARKER;
			break;
		case BASETYPE_INHERITANCE_MARKER:
			markerName = BASETYPE_INHERITANCE_MARKER;
			break;
		case CYCLIC_INHERITANCE_MARKER:
			markerName = CYCLIC_INHERITANCE_MARKER;
			break;
		case DOUBLE_ATTRIBUTENAME_MARKER:
			markerName = DOUBLE_ATTRIBUTENAME_MARKER;
			break;
		case INVALID_ATTRIBUTE_MODIFIER_MARKER:
			markerName = INVALID_ATTRIBUTE_MODIFIER_MARKER;
			break;
		case INVALID_OVERLOADING_MARKER:
			markerName = INVALID_OVERLOADING_MARKER;
			break;
		case INVALID_PRODUCT_MARKER:
			markerName = INVALID_PRODUCT_MARKER;
			break;
		case PARSER_MARKER:
			markerName = PARSER_MARKER;
			break;
		case REFERENCER_MARKER:
			markerName = REFERENCER_MARKER;
			break;
		default:
			markerName = UNDEFINED_MARKER;
			break;
		}
		
		IMarker marker;
		try {
			marker = file.createMarker(markerName);
			marker.setAttribute(IMarker.LINE_NUMBER, line);
			marker.setAttribute(IMarker.CHAR_START, start);
			marker.setAttribute(IMarker.CHAR_END, end);
			marker.setAttribute(IMarker.MESSAGE, message);
			marker.setAttribute(IMarker.TEXT, message);
			marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
		} catch (final CoreException e) {
			e.printStackTrace();
		}
		
	}
	
	public static void writeUnknownErrorMarker(final IResource file, final String message) {
		
		IMarker marker;
		try {
			marker = file.createMarker(UNDEFINED_MARKER);
			marker.setAttribute(IMarker.MESSAGE, message);
			marker.setAttribute(IMarker.TEXT, message);
			marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
		} catch (final CoreException e) {
			e.printStackTrace();
		}
		
	}
	
	/**
	 * This Method removes all written Markers on a given File.
	 * 
	 * @param file
	 *            The File where the Markers will be removed from. Must implement IResource.
	 */
	public static void releaseAllParserMarkersOnFile(final IResource file) {
		try {
			file.deleteMarkers(ABSTRACT_OPERATIONS_MARKER, true, IResource.DEPTH_INFINITE);
			file.deleteMarkers(BASETYPE_INHERITANCE_MARKER, true, IResource.DEPTH_INFINITE);
			file.deleteMarkers(CYCLIC_INHERITANCE_MARKER, true, IResource.DEPTH_INFINITE);
			file.deleteMarkers(DOUBLE_ATTRIBUTENAME_MARKER, true, IResource.DEPTH_INFINITE);
			file.deleteMarkers(INVALID_ATTRIBUTE_MODIFIER_MARKER, true, IResource.DEPTH_INFINITE);
			file.deleteMarkers(INVALID_OVERLOADING_MARKER, true, IResource.DEPTH_INFINITE);
			file.deleteMarkers(INVALID_PRODUCT_MARKER, true, IResource.DEPTH_INFINITE);
			file.deleteMarkers(PARSER_MARKER, true, IResource.DEPTH_INFINITE);
			file.deleteMarkers(REFERENCER_MARKER, true, IResource.DEPTH_INFINITE);
			file.deleteMarkers(UNDEFINED_MARKER, true, IResource.DEPTH_INFINITE);
		} catch (final CoreException e) {
			e.printStackTrace();
		}
	}
	
}
